use std::time::Duration;

use aspotify::PlaylistItemType::{Episode, Track};
use aspotify::{Client, ClientCredentials, TrackSimplified};

pub enum SpotifyKind {
    Track(String),
    Album(String),
    Playlist(String),
}

fn get_id_in_url(url: &str) -> Option<&str> {
    url.rsplit('/')
        .next()
        .and_then(|x| x.split(' ').next())
        .and_then(|x| x.split('?').next())
}

pub fn get_spotify_entry(url: &str) -> Option<SpotifyKind> {
    if url.contains("https://open.spotify.com/track/") {
        let track_id = get_id_in_url(url);
        return match track_id {
            Some(id) => Some(SpotifyKind::Track(id.to_string())),
            None => None,
        };
    }
    if url.contains("https://open.spotify.com/album/") {
        let album_id = get_id_in_url(url);
        return match album_id {
            Some(id) => Some(SpotifyKind::Album(id.to_string())),
            None => None,
        };
    }
    if url.contains("https://open.spotify.com/playlist/") {
        let playlist_id = get_id_in_url(url);
        return match playlist_id {
            Some(id) => Some(SpotifyKind::Playlist(id.to_string())),
            None => None,
        };
    }
    return None;
}

pub fn get_spotify_client() -> Box<Client> {
    let spotify_creds =
        ClientCredentials::from_env().expect("CLIENT_ID and CLIENT_SECRET not found.");
    let spotify_client = Box::new(Client::new(spotify_creds));
    spotify_client
}

pub struct TrackInfo {
    pub(crate) name: String,
    pub(crate) artists: Vec<String>,
    pub(crate) duration: Duration,
}

pub struct AlbumInfo {
    pub(crate) name: String,
    pub(crate) artists: Vec<String>,
    pub(crate) genres: Vec<String>,
    pub(crate) tracks: Vec<TrackInfo>,
}

pub struct PlaylistInfo {
    pub(crate) name: String,
    pub(crate) artists: Vec<String>,
    pub(crate) tracks: Vec<TrackInfo>,
}

pub async fn get_spotify_track(spotify: Box<Client>, id: &String) -> Option<TrackInfo> {
    match spotify.tracks().get_track(id.as_str(), None).await {
        Ok(track) => Some(TrackInfo {
            name: track.data.name,
            artists: track.data.artists.iter().map(|x| x.name.clone()).collect(),
            duration: track.data.duration,
        }),
        Err(_e) => None,
    }
}

pub async fn get_spotify_album(spotify: Box<Client>, id: &String) -> Option<AlbumInfo> {
    match spotify.albums().get_album(id.as_str(), None).await {
        Ok(album) => Some(AlbumInfo {
            name: album.data.name,
            artists: album.data.artists.iter().map(|x| x.name.clone()).collect(),
            genres: album.data.genres,
            tracks: album
                .data
                .tracks
                .items
                .iter()
                .map(|t| TrackInfo {
                    name: t.name.clone(),
                    artists: t.artists.iter().map(|x| x.name.clone()).collect(),
                    duration: t.duration,
                })
                .collect(),
        }),
        Err(_e) => None,
    }
}

fn extract_artists_from_tracks(tracks: Vec<TrackSimplified>) -> Vec<String> {
    tracks
        .iter()
        .flat_map(|t| t.artists.iter().map(|a| a.name.clone()))
        .collect()
}

pub async fn get_spotify_playlist(spotify: Box<Client>, id: &String) -> Option<PlaylistInfo> {
    match spotify.playlists().get_playlist(id.as_str(), None).await {
        Ok(playlist) => Some(PlaylistInfo {
            name: playlist.data.name,
            artists: playlist
                .data
                .tracks
                .items
                .iter()
                .flat_map(|p| {
                    p.item.as_ref().map_or(Vec::new(), |x| match x {
                        Track(t) => extract_artists_from_tracks(vec![t.clone().simplify()]),
                        Episode(_e) => Vec::new(),
                    })
                })
                .collect(),
            tracks: playlist
                .data
                .tracks
                .items
                .iter()
                .flat_map(|p| {
                    p.item.as_ref().map_or(None, |x| match x {
                        Track(t) => Some(TrackInfo {
                            name: t.name.clone(),
                            artists: extract_artists_from_tracks(vec![t.clone().simplify()]),
                            duration: t.duration,
                        }),
                        Episode(_e) => None,
                    })
                })
                .collect(),
        }),
        Err(_e) => None,
    }
}
